/** @file  ds1307.c
 *  @brief Gestion RTC DS1307
 */
#include "ds1307.h"
#include "i2c.h"
#include "tempo.h"
#include "rs232.h"


//========================================================================================
// variables externes
//========================================================================================
extern u8 hh, mm, ss;
extern u8 mois, jour, annee, jourSemaine;

//========================================================================================
// DS1307 RTC
//========================================================================================
unsigned char
bcd2hex(unsigned char val) 
{
	return ((val & 0x0F) + ((val>>4)&0x0F)*10);
}

unsigned char
hex2bcd(unsigned char val) 
{
	return ((val/10)<<4) + (val % 10);
}

//---------------------------------------------------------------------------
/** Initialisation RTC avec RAZ secondes
 \param 
 */
//---------------------------------------------------------------------------
void
ds1307_init(void) {
	unsigned char ch;

	#ifdef DEBUG_RS232
		tx232_puts("1307init\nr");
	#endif

	i2c_start();
	i2c_write(DS1307_ADR | I2C_WRITE);
	i2c_write(0x00);			// adr 0
	i2c_stop();

	i2c_start();
	i2c_write(DS1307_ADR | I2C_READ);
	ch = i2c_read(0);			// nack
	i2c_stop();

	// si RTC OFF
	if (ch&0x80) 
	{
		i2c_start();
		i2c_write(DS1307_ADR | I2C_WRITE);
		i2c_write(0x00);				// adr 0
		i2c_write(0x00);				// CH = 0 : enable osc
		i2c_stop();
	}
}

//---------------------------------------------------------------------------
/** Reglage de l'heure + date RTC 
 \param 
 */
//---------------------------------------------------------------------------
void
ds1307_set_time(void) {
	unsigned char tmp;	

	#ifdef DEBUG_RS232
		tx232_puts("1307set\nr");
	#endif
		
	i2c_start();
	i2c_write(DS1307_ADR | I2C_WRITE);
	i2c_write(0x00);					// adr 0
	i2c_write(hex2bcd(ss) & ~0x80);	// CH = 0 : enable osc
	i2c_write(hex2bcd(mm));			// 
	i2c_write(hex2bcd(hh));			// 
	tmp = jourSemaine+1;
	if (jourSemaine==8) jourSemaine = 0;	
	i2c_write(hex2bcd(jourSemaine+1));	// 
	i2c_write(hex2bcd(jour));		// 
	i2c_write(hex2bcd(mois));		// 
	i2c_write(hex2bcd(annee));		// 
	i2c_write(0x00);					// config SQW OFF
	i2c_stop();
}

//---------------------------------------------------------------------------
/** Lecture heure + date RTC
 \param 
 */
//---------------------------------------------------------------------------
void
ds1307_get_time(void) 
{

	#ifdef DEBUG_RS232
		tx232_puts("1307get\nr");
	#endif
		
	i2c_start();
	i2c_write(DS1307_ADR | I2C_WRITE);
	i2c_write(0x00);			// adr 0
	i2c_stop();

	DelayBigMs(1);

	i2c_start();
	i2c_write(DS1307_ADR | I2C_READ);

	// secondes
	ss = bcd2hex( i2c_read(1) );	
	
	// minutes
	mm = bcd2hex( i2c_read(1) );
	
	// heures
	hh = bcd2hex( i2c_read(1) & 0x3F);	

	// jour de la semaine (1..7)
	jourSemaine = i2c_read(1) - 1;
	if (jourSemaine==7) jourSemaine=0;		// 0 = Dimanche

	// jour
	jour = bcd2hex( i2c_read(1) );	

	// mois
	mois = bcd2hex( i2c_read(1) );	

	// annee
	annee = bcd2hex( i2c_read(0) );			// nack

	i2c_stop();
}
