/** @file i2c.c
 */
#include "i2c.h"
#include "rs232.h"

//========================================================================================
/* BUS I2C */
// SSPADD = ((Fosc/Fi2c)-1)/4
//========================================================================================
void
i2c_init(void) 
{
	char x;
	
	// set SCL and SDA pins as inputs
	TRISCbits.TRISC3=1;	
	TRISCbits.TRISC4=1;	
	
	SSPCON1 = 0x38;					// set I2C master mode
	SSPCON2 = 0x00;					
	//SSPADD = 0x0E;						// Fi2c= Fosc/(4*(SSPADD+1))   ; 0x0C 400kHz bus with 20MHz xtal
	// 100KHz a 40MHz 
	///SSPADD = 0x63;						// Fi2c= Fosc/(4*(SSPADD+1))   ; 0x0C 400kHz bus with 20MHz xtal
	// 0x1B
	SSPADD = ((FOSC/FI2C)-1)/4;
	
	SSPSTATbits.CKE = 0;								// use I2C levels      worked also with '0'
	SSPSTATbits.SMP = 0;								// disable slew rate control  worked also with '0'

	x = SSPBUF;			// dummy read clears BF

	PIR1bits.SSPIF  = 0;								// clear SSPIF interrupt flag 
	PIR2bits.BCLIF  = 0;								// clear bus collision flag
}


//========================================================================================
void
i2c_waitForIdle(void) 
{
	// wait for idle and not writing
	while ( ((SSPCON2 & 0x1F)!=0) || SSPSTATbits.R_W ) 
	{
		
	}

////	while (( SSPCON2 & 0x1F ) | (SSPSTAT & 0x04)) 

}


//========================================================================================
void
i2c_start(void) 
{
	#ifdef DEBUG_RS232	
		tx232('S');
	#endif

	i2c_waitForIdle();
	SSPCON2bits.SEN = 1;
}


//========================================================================================
void
i2c_repStart(void) 
{
	#ifdef DEBUG_RS232
		tx232('R');
	#endif
	i2c_waitForIdle();
	DelayUs(5);		    //wait 12 uS  (Running @ 20 MHz)
	SSPCON2bits.RSEN = 1;
}


//========================================================================================
void
i2c_stop(void)
{
	
#ifdef DEBUG_RS232
	tx232('P');
#endif	
	i2c_waitForIdle();
	DelayUs(5);		    //wait 12 uS  (Running @ 20 MHz)
	SSPCON2bits.PEN = 1;
}


//========================================================================================
int 
i2c_read( unsigned char ack )
{
	unsigned char i2cReadData;
	i2c_waitForIdle();
	DelayUs(5);		    //wait 12 uS  (Running @ 20 MHz)
	SSPCON2bits.RCEN = 1;
	i2c_waitForIdle();
	i2cReadData = SSPBUF;
	i2c_waitForIdle();
	if (ack)	SSPCON2bits.ACKDT = 0;
	else		SSPCON2bits.ACKDT = 1;
	SSPCON2bits.ACKEN = 1;               			// send acknowledge sequence

	#ifdef DEBUG_RS232
		tx232('R');
		tx232_2ascii(i2cReadData);
	#endif
	
	return( i2cReadData );
}


//========================================================================================
unsigned char 
i2c_write( unsigned char i2cWriteData )
{
	i2c_waitForIdle();
	DelayUs(5);		    //wait 12 uS  (Running @ 20 MHz)
	SSPBUF = i2cWriteData;

	#ifdef DEBUG_RS232
		tx232('W');
		tx232_2ascii(i2cWriteData);
	#endif
	
	return ( ! (SSPCON2bits.ACKSTAT)  ); 			// function returns '1' if transmission is acknowledged
}

/*
//========================================================================================
void
rw_start_eeprom(unsigned int address)
{
	i2c_start();
	i2c_write(0xa0);
	i2c_write(address >> 8);
	i2c_write(address & 0x00FF);	
}


//========================================================================================
void
write_ext_eeprom(unsigned int address, unsigned char data)
{
	rw_start_eeprom(address);
	i2c_write(data);
	i2c_stop();
	DelayMs(11);
}

//========================================================================================
unsigned char 
read_ext_eeprom(unsigned int address)
{
	unsigned char data;
	rw_start_eeprom(address);	
	i2c_repStart();
	i2c_write(0xa1);
	data = i2c_read(0);				// nack (1 data  lire)
	i2c_stop();
	return(data);
}
*/
