/** @file  lm92.c
 *  @brief LM92 I2C temperature sensor 
 */
#include "lm92.h"
#include "i2c.h"
#include "rs232.h"

/** @brief Get LM92 temperature register
 *  @return : temperature 29 bits (1 bit = 0.0625C)
 */
s16 
lm92_read_temperature()
{
	s16 temperature = 0;

	#ifdef DEBUG_RS232
		tx232_puts("lm92read\nr");
	#endif

	
	// Set pointer to 0
	
	
	// Read value
	i2c_start();
	i2c_write(LM92_ADR | I2C_READ);
	
	// MSB ack
	temperature  = i2c_read(1) << 8;	

	// LSB no ack
	temperature |= i2c_read(0);		
	
	// remove 3 LSB bits (crit, high, low)
	temperature>>=3;
	i2c_stop();
	
	return temperature;
}


s8 
lm92_get_signed_temperature()
{
	s16 temp = 	lm92_read_temperature()>>4;	// divide by 16 (1/0.0625)
	if (temp>99) temp=99;
	if (temp<=-99) temp=0;
	return (s8)temp;
}

