//----------------------------------------------------------------------------
/** @file main.c
 *  @brief Horloge style Radio
 *  SDCC 3.3 mini
 */
// Project : HorloRadio
//----------------------------------------------------------------------------
// Includes files
//----------------------------------------------------------------------------
#include <pic18fregs.h>
#include <pic18f452.h>
#include <stdio.h>
#include <signal.h>	// for interrupt

 
#include "tempo.h"
#include "i2c.h"
#include "ds1307.h"
#include "lm92.h"

#ifdef DEBUG_RS232			
	#include "rs232.h"
	#include "stdio.h" // for printf
#endif

//----------------------------------------------------------------------------
// Configuration Register : FUSES
//----------------------------------------------------------------------------
#include "port.h"

// sdcc 3.3.0 uses pragma (see gpasm p18c452.inc)
//#pragma config CP = OFF
// HS w/PLL Enabled, Clk Freq = 4xFreq Osc.
#pragma config OSC = HSPLL
#pragma config OSCS = OFF
#pragma config PWRT = ON
// brown out reset disabled
#pragma config BOR = OFF
// watch dog timer disabled
#pragma config WDT = OFF
// low voltage programming disabled
#pragma config LVP = OFF
#pragma config STVR = OFF
// debug disabled
#pragma config DEBUG = OFF


//========================================================================================
// Variables globales
//========================================================================================
#define MODE_TIME		(0)
#define MODE_TIME_TEMP	(1)
#define MODE_MAX		(MODE_TIME_TEMP)
u8 mode;

// 7 segment display
u8 aff_on;
u8 aff[4];
#define AFF_MAX 4


__code const u8 Aff_chiffre[] = 
{
	~(AFF_A | AFF_B | AFF_C | AFF_D | AFF_E | AFF_F),		// 0
	~(AFF_B | AFF_C), 										// 1
	~(AFF_A | AFF_B | AFF_D | AFF_E | AFF_G), 				// 2
	~(AFF_A | AFF_B | AFF_C | AFF_D | AFF_G), 				// 3
	~(AFF_B | AFF_C | AFF_F | AFF_G), 						// 4
	~(AFF_A | AFF_C | AFF_D | AFF_F | AFF_G), 				// 5
	~(AFF_C | AFF_D | AFF_E | AFF_F | AFF_G), 				// 6
	~(AFF_A | AFF_B | AFF_C), 								// 7
	~(AFF_A | AFF_B | AFF_C | AFF_D | AFF_E | AFF_F | AFF_G), // 8
	~(AFF_A | AFF_B | AFF_C | AFF_F | AFF_G), 				// 9

	~(AFF_A | AFF_B | AFF_C | AFF_E | AFF_F | AFF_G), 		// A	// 10
	~(AFF_C | AFF_D | AFF_E | AFF_F | AFF_G), 				// B	// 11
	~(AFF_A | AFF_D | AFF_E | AFF_F), 						// C	// 12
	~(AFF_B | AFF_C | AFF_D | AFF_E | AFF_G), 				// D	// 13
	~(AFF_A | AFF_D | AFF_E | AFF_F | AFF_G), 				// E	// 14
	~(AFF_A | AFF_E | AFF_F | AFF_G), 						// F	// 15
	~(AFF_G),												// -	// 16
	~(AFF_A | AFF_B | AFF_F | AFF_G),						// 	// 17
	~(0),													// empty	// 18
};
#define CHAR_A (10)
#define CHAR_B (11)
#define CHAR_C (12)
#define CHAR_D (13)
#define CHAR_E (14)
#define CHAR_F (15)
#define CHAR_MINUS		(16)
#define CHAR_DEG		(17)
#define CHAR_SPACE		(18)


u16 tmr0_count;
u8 hh, mm, ss;
u8 mois, jour, annee, jourSemaine;
s8 temperature;

typedef union
{
	struct
	{
		unsigned buzzer:1;
		unsigned buttonDebounce:1;
		unsigned dummy1:7;
	};
}type_status;

type_status bit_status;



//----------------------------------------------------------------------------
// Interrupt handler
//----------------------------------------------------------------------------
DEF_INTHIGH(high_int)
DEF_HANDLER(SIG_TMR0, _tmr0_handler)
END_DEF

//SIG_RB PORTB change interrupt
//SIG_INT0 INT0 external interrupt
//SIG_INT1 INT1 external interrupt
//SIG_INT2 INT2 external interrupt
//SIG_CCP1 CCP1 module interrupt
//SIG_CCP2 CCP2 module interrupt
//SIG_TMR0 TMR0 overflow interrupt
//SIG_TMR1 TMR1 overflow interrupt
//SIG_TMR2 TMR2 matches PR2 interrupt
//SIG_TMR3 TMR3 overflow interrupt
//SIG_EE EEPROM/FLASH write complete interrupt
//SIG_BCOL Bus collision interrupt
//SIG_LVD Low voltage detect interrupt
//SIG_PSP Parallel slave port interrupt
//SIG_AD AD convertion complete interrupt
//SIG_RC USART receive interrupt
//SIG_TX USART transmit interrupt
//SIG_MSSP SSP receive/transmit interrupt

#define TMR0_COUNT 1350

SIGHANDLER(_tmr0_handler)
{
	u8 new_sec = 0;
	// action to be taken when timer 0 overflows 
	tmr0_count--;
	if (tmr0_count==-1)
	{
		tmr0_count = TMR0_COUNT;
		ss++;
		new_sec = 1;
		if (ss>=60) 
		{
			ss = 0;
			mm++;
		}
		if (mm>=60) 
		{
			mm = 0;
			hh++;
		}
		if (hh>=24) 
		{
			hh=0;
		}
	}

	
	// Set display value : default time
	aff[0] = Aff_chiffre[hh>=10 ? (hh/10) : CHAR_SPACE];
	aff[1] = Aff_chiffre[hh%10];
	aff[2] = Aff_chiffre[mm/10];
	aff[3] = Aff_chiffre[mm%10];
			
	switch(mode)
	{
		case MODE_TIME :
			if (new_sec) LED_SEC = !LED_SEC;
			break;
		case MODE_TIME_TEMP	:
			// 4s temperature, 4s time
			if ((ss>>2)%2)	// time/4
			//if (ss%2)	// test
			{
				if (new_sec) LED_SEC = !LED_SEC;
				break;
			}
			LED_SEC	= 0;
			if (temperature>=0)
			{
				aff[0] = Aff_chiffre[CHAR_SPACE];
				aff[1] = Aff_chiffre[temperature>=10 ? (temperature/10) : CHAR_SPACE];
				aff[2] = Aff_chiffre[temperature%10];
			}
			else
			{
				aff[0] = Aff_chiffre[CHAR_MINUS];
				aff[1] = Aff_chiffre[temperature<=-10 ? (-temperature/10) : CHAR_SPACE];
				aff[2] = Aff_chiffre[-temperature%10];
			}
			aff[3] = Aff_chiffre[CHAR_DEG];
			break;

		default :
			break;
	}

	
	// Display 7 segment clock
	aff_on++;
	if (aff_on>=AFF_MAX) 
	{
		aff_on = 0;
	}

	AFF_PORT = 0;	
	SET_CHIFFRE(aff_on)
	AFF_PORT = aff[aff_on];


	// Buzzer
	if (bit_status.buzzer)
	{
		BUZ = !BUZ;	
	}
	else
	{
		BUZ = 0;	
	}

	// Clear interrupt flag;
	INTCONbits.T0IF = 0;
	
	// Set rollover counter
	//TMR0L = ;
	//TMR0H = ;
}

//========================================================================================
/**  initialisation Timer0
 *   FOSC/4 => Timer0 => Prescaler 1/64 => 256 counter =>  TMR0IF
 *   Quartz : 11.0592MHz  
 *   Fclk = FOSC/4*PLL = 11.0592MHz/4*4 = 11.0592MHz
 *   PrescalerTMR0 = 32 (mode 8 bits=256)
 *   1350 * 32 * 256 * TCLK = 1 seconde
 */
//========================================================================================
void
init_timer0(void)
{
	// set timer0
	T0CON = 0;

	tmr0_count = TMR0_COUNT;
	
//	TMR0L = TMR0_COUNT&0xFF;
//	TMR0H = TMR0_COUNT>>8;
	TMR0L = 1;
	TMR0H = 0;	
	
	T0CONbits.T08BIT = 1;	// 8 bits counter(1) (0=16bits)
	//T0CONbits.T0CS = 0;		// CLK0
	//T0CONbits.PSA = 0;		// PRESCALER On

#if 0
	//	prescaler 1/32
	T0CONbits.T0PS0 = 0;
	T0CONbits.T0PS1 = 0;
	T0CONbits.T0PS2 = 1;
#else
	//	prescaler 1/8 (bug HS PLL * 4 ??)
	T0CONbits.T0PS0 = 0;
	T0CONbits.T0PS1 = 1;
	T0CONbits.T0PS2 = 0;
#endif
	
	// enable timer0
	T0CONbits.TMR0ON = 1;

	// enable timer0 ISR
	INTCONbits.T0IE = 1;
	
	// Global ISR ON
	INTCONbits.GIE = 1;	
}


//========================================================================================
/**  initialisation des parametres
 */
//========================================================================================
void
init_system(void){
	//=============== INITIALISATION DES PORTS I/O ===============//
	PORTA = 0;								// Sorties portA a 0
	PORTB = 0;								// sorties portB a 0
//	PORTC = 0;								// sorties portC a 0	!! NE PAS METTRE A ZERO RC6/7 UART
	PORTC &= 0xC0;
	PORTD = 0;								// sorties portD a 0
	PORTE = 0;								// sorties portE a 0
	
//	ADCON0 = 0x85;							// 0b10000101 (Fosc/32, RA0, GO, 0, A/D on)
//	ADCON1 = 0b10001110;					// (right justify (10bits), RA0 ana)

// RA0 ana
//	ADCON0 = 0xC5;							// RC, RA0, GO, A/D on
//	ADCON1 = 0x0E;							// 0b00001110 (left justify (8bits), RA0 ana)

	ADCON0 = 0xC1;							// RC, canal 0, GO, A/D on
	ADCON1 = 0x44;							// left justify (8bits), RC, RA3,1,0 ANA

	TRISA = TRISACONF;	 				// '1' entree ; '0' sortie
	TRISB = TRISBCONF;					// '1' entree ; '0' sortie
	TRISC = TRISCCONF;	 				// '1' entree ; '0' sortie
	TRISD = TRISDCONF;	 				// '1' entree ; '0' sortie
	TRISE = TRISECONF;	 				// '1' entree ; '0' sortie


	//========= CONFIGURATION DES INTERRUPTIONS ==========//
	//PIE1 = 0b00100001; 					// RCIE (usart RX) + TIMER1
	//PIE2 = 0b00000000;
	//RCIF = 0;
	//INTCON = 0b01000000; 				// masque interruption, GIE + PIE

	//========= INITIALISATION DES PERIPHERIQUES =========//
	
	/////////// DEBUG RS232 ///////////
	#ifdef DEBUG_RS232
		init232(DIVIDER);						// port serie, 115kbds, pas de parite, 8bits, 1 stop
		tx232_puts("horloge radio\n\r");
	//	stdout = STREAM_USER; // this is not necessery, since stdout points by default to STREAM_USER
	//	printf("test printf div=%d\n", 99);
	#endif

	
	i2c_init();								// initialisation Bus I2C
	
	// wait I2C
	DelayBigMs(1);
	
	// Initialize RTC
	ds1307_init();					// init RTC
	
	//GIE = 1;									// interruptions ON
}


//----------------------------------------------------------------------------
/** 60 Leds control
 */
//----------------------------------------------------------------------------
led_sec_type led_sec; 


//----------------------------------------------------------------------------
/** Set Latches 4094
 */
//----------------------------------------------------------------------------
void
set_led(void)
{
	u8 bitData = 0x1;
	u8 idx, regId;
	s8 i;
	REG_STR   = 0;
	REG_CLK   = 0;

	for(idx=0, regId=7; idx<8; regId--, idx++)
	{
		for(i=3; i>=0; i--)
		{
			bitData = 1<<i;
			if (led_sec.reg[regId] & bitData)
			{
				REG_DATA  = 0;
			}
			else
			{
				REG_DATA  = 1;
			}
			
			REG_CLK   = 1;
			REG_CLK   = 0;
		}	

		for(i=4; i<8; i++)
		{
			bitData = 1<<i;
			if (led_sec.reg[regId] & bitData)
			{
				REG_DATA  = 0;
			}
			else
			{
				REG_DATA  = 1;
			}
			
			REG_CLK   = 1;
			REG_CLK   = 0;
			
		}	
	}

	REG_STR   = 1;
	REG_STR   = 0;
}


//----------------------------------------------------------------------------
/** Set led on into memory
 *  @param second : led to set ON
 */
//----------------------------------------------------------------------------
void
set_led_bit(u8 second)
{
	u8 reg, led;
	
	if (second>=60) second = 0;
	
	reg = second / 8;
	if (reg>=8) reg = 0;
	
	led = second%8;
	
	led_sec.reg[reg] |= (1<<led);
}


//----------------------------------------------------------------------------
/** Set all led memory
 *  @param value : value to set on each latch
 */
//----------------------------------------------------------------------------
void
memset_led_all(u8 value)
{
	led_sec.reg[0] = value;
	led_sec.reg[1] = value;
	led_sec.reg[2] = value;
	led_sec.reg[3] = value;
	led_sec.reg[4] = value;
	led_sec.reg[5] = value;
	led_sec.reg[6] = value;
	led_sec.reg[7] = value;
}


typedef enum
{
	MODE_SEC_NORMAL,
	MODE_SEC_REVERSE,
	MODE_SEC_POINT	
}ModeSeconde;


//----------------------------------------------------------------------------
/** Set leds with mode
 *  @param second : value of current second
 *  @param mode   : mode to display (Normal, reverse, 1 led on)
 */
//----------------------------------------------------------------------------
void
set_led_clock_second_mode(u8 second, ModeSeconde mode)
{
	u8 i;
	
	// clear all leds
	memset_led_all(0x00);	
	
	switch(mode)
	{
		case MODE_SEC_NORMAL :
			for(i=0;i<second;i++)
			{
				set_led_bit(i+1);
			}
			break;	
			
		case MODE_SEC_REVERSE :
			for(i=second;i<60;i++)
			{
				set_led_bit(i+1);
			}
			break;	

		case MODE_SEC_POINT :
			set_led_bit(second+1);
			break;	
	}

	// set latch
	set_led();
}


//----------------------------------------------------------------------------
/** Set leds
 */
//----------------------------------------------------------------------------
void
set_led_clock_second()
{
	u8 ss_tmp = ss;
	
	if (mm%2)
	{
		set_led_clock_second_mode(ss_tmp, MODE_SEC_NORMAL);
	}
	else
	{
		set_led_clock_second_mode(ss_tmp, MODE_SEC_REVERSE);
	}	
}



#ifdef DEBUG_RS232
	char getchar(void)
	{
		return rx232_detc();	
	}
	
	/*void putchar(unsigned char val)
	{
		tx232(val);	
	}*/
	PUTCHAR(c)
	{
		tx232(c);
	}
#endif


//----------------------------------------------------------------------------
/** Main program
 */
//----------------------------------------------------------------------------
void
main(void)
{
	//u8 i, j, val, chiffre; 
	bit_status.buttonDebounce = 0;
	
	// initialize system I/O
	init_system();

	mode = MODE_TIME_TEMP;
	
	hh     = 12;
	mm     =  0;
	ss     =  0;
	mois   =  1;
	jour   =  1;
	jourSemaine = JOUR_MAR;
	annee  =  13;	// 2013
	
	temperature = lm92_get_signed_temperature();
	aff_on = 0;
	bit_status.buzzer = 0;

	// init timer0 for clock and global ISR
	init_timer0();


	// Reset clock
	if (!BP1 && !BP2 && !BP3)
	{
		u8 cpt;
		ds1307_set_time();

		for(cpt=0; cpt<4; cpt++)
		{
			bit_status.buzzer = 1;
			DelayBigMs(10);
			bit_status.buzzer = 0;
		}		
	}
	
	// Get time from RTC
	ds1307_get_time();			// recupere heure + date actuelle


	// main loop
	for(;;)
	{
		// display LEDs
		set_led_clock_second();
		
		// button
		if (BP_READ)
		{
			Button_type btn, btn2;
			btn.value = BP_READ;
			DelayBigMs(20);
			btn2.value = BP_READ;
			
			if (btn.value == btn2.value)
			{
				switch(btn.value)
				{
					case BP1_MASK :	
						if (mode!=MODE_TIME)
							mode = MODE_TIME;
						else
						{
							INTCONbits.GIE = 0;
							if (++hh>=24) hh = 0;
							INTCONbits.GIE = 1;
							ds1307_set_time();
						}
						break;
					
					case BP2_MASK :	
						if (mode!=MODE_TIME)
							mode = MODE_TIME;
						else
						{
							INTCONbits.GIE = 0;
							if (++mm>=60) mm = 0;
							INTCONbits.GIE = 1;
							ds1307_set_time();
						}
						break;
					
					case BP3_MASK :	
						// buzzer
						/*bit_status.buzzer = 1;
						DelayBigMs(255);
						bit_status.buzzer = 0;

						ds1307_set_time();*/
						
						// change mode
						if (++mode>MODE_MAX) mode = 0;
						break;
				}	
				bit_status.buttonDebounce = 1;			
			}			
		}
		
		if ((ss%32)==0)
		{
			temperature = lm92_get_signed_temperature();
		}

		// debounce button
		if (bit_status.buttonDebounce)
		{
			while(BP_READ)
			{
				// display LEDs
				set_led_clock_second();
			}
			bit_status.buttonDebounce = 0;
		}		
		
	
	}	
}

	
