//----------------------------------------------------------------------------
/** @file port.h
 *  @brief Input ouput port definition
 */
//----------------------------------------------------------------------------
#ifndef PORT_H
#define PORT_H

#include "pic18f452.h"

#define FOSC 44236800L 		// 11.0592 MHz  HSPLL x4
//#define FOSC 11059200L 		// 11.0592 MHz  HSPLL x1

#define DEBUG_RS232			// debug message on RS232

//----------------------------------------------------------------------------
// Types
//----------------------------------------------------------------------------
typedef unsigned char u8;
typedef signed char   s8;
typedef unsigned int  u16;
typedef signed int    s16;

//----------------------------------------------------------------------------
// Push button
//----------------------------------------------------------------------------
#define BP1       PORTCbits.RC5
#define BP2       PORTCbits.RC6
#define BP3       PORTCbits.RC7
#define BP_PORT	PORTC
#define BP_MASK	0xE0
#define BP1_MASK	(1<<5)
#define BP2_MASK	(1<<6)
#define BP3_MASK	(1<<7)
#define BP_READ	(~PORTC & 0xE0)

typedef union
{
	union
	{
		unsigned dummy : 5;
		unsigned bp1 : 1;
		unsigned bp2 : 1;
		unsigned bp3 : 1;
	}bits;
	u8 value;	
}Button_type;

//----------------------------------------------------------------------------
// Buzzer
//----------------------------------------------------------------------------
#define BUZ       PORTCbits.RC1

//----------------------------------------------------------------------------
// 4094 registers
//----------------------------------------------------------------------------
#define REG_STR   PORTEbits.RE0
#define REG_DATA  PORTEbits.RE1
#define REG_CLK   PORTEbits.RE2


typedef struct
{
	union
	{
		u8 d7 : 1;
		u8 d6 : 1;
		u8 d5 : 1;
		u8 d4 : 1;
		u8 d0 : 1;
		u8 d1 : 1;
		u8 d2 : 1;
		u8 d3 : 1;
		
		u8 d15 : 1;
		u8 d14: 1;
		u8 d13 : 1;
		u8 d12 : 1;
		u8 d8 : 1;
		u8 d9 : 1;
		u8 d10 : 1;
		u8 d11 : 1;
	
		u8 d23 : 1;
		u8 d22 : 1;
		u8 d21 : 1;
		u8 d20 : 1;
		u8 d16 : 1;
		u8 d17 : 1;
		u8 d18 : 1;
		u8 d19 : 1;
	
		u8 d31 : 1;
		u8 d30 : 1;
		u8 d29 : 1;
		u8 d28 : 1;
		u8 d24 : 1;
		u8 d25 : 1;
		u8 d26 : 1;
		u8 d27 : 1;
	
		u8 d39 : 1;
		u8 d38 : 1;
		u8 d37 : 1;
		u8 d36 : 1;
		u8 d32 : 1;
		u8 d33 : 1;
		u8 d34 : 1;
		u8 d35 : 1;
	
		u8 d47 : 1;
		u8 d46 : 1;
		u8 d45 : 1;
		u8 d44 : 1;
		u8 d40 : 1;
		u8 d41 : 1;
		u8 d42 : 1;
		u8 d43 : 1;
	
		u8 d55 : 1;
		u8 d54 : 1;
		u8 d53 : 1;
		u8 d52 : 1;
		u8 d48 : 1;
		u8 d49 : 1;
		u8 d50 : 1;
		u8 d51 : 1;
	
		u8 dx0 : 1;
		u8 dx1 : 1;
		u8 dx2 : 1;
		u8 dx3 : 1;
		u8 d56 : 1;
		u8 d57 : 1;
		u8 d58 : 1;
		u8 d59 : 1;
	} bits;
 	u8 reg[8];	//!< 8 registers
 	
}led_sec_type;


//----------------------------------------------------------------------------
// Led control
//----------------------------------------------------------------------------
#define LED_SEC   PORTBbits.RB5

#define LED_AFF1   PORTBbits.RB3
#define LED_AFF2   PORTBbits.RB1
#define LED_AFF3   PORTBbits.RB2
#define LED_AFF4   PORTBbits.RB4

enum
{
	AFF_A = 0x80,
	AFF_B = 0x20,
	AFF_C = 0x4, 
	AFF_D = 0x2, 
	AFF_E = 0x10,
	AFF_F = 0x40, 
	AFF_G = 0x8,
};


#define AFF_PORT  PORTD

#define OUTPUT_CHIFFRE(nb) AFF_PORT = Aff_chiffre[nb]


#define SET_CHIFFRE(chiffre) \
	{\
		LED_AFF1 = 0; LED_AFF2 = 0; LED_AFF3 = 0; LED_AFF4 = 0; \
		switch(chiffre) \
		{ \
			default : \
			case 0: LED_AFF1 = 1; break; \
			case 1: LED_AFF2 = 1; break; \
			case 2: LED_AFF3 = 1; break; \
			case 3: LED_AFF4 = 1; break; \
		} \
	}



#define SET_AFF1(nb) \
	{ \
		LED_AFF1 = 1; LED_AFF2 = 0; LED_AFF3 = 0; LED_AFF4 = 0; \
		OUTPUT_CHIFFRE(nb); \
	}

#define SET_AFF2(nb) \
	{ \
		LED_AFF1 = 0; LED_AFF2 = 1; LED_AFF3 = 0; LED_AFF4 = 0; \
		OUTPUT_CHIFFRE(nb); \
	}

#define SET_AFF3(nb) \
	{ \
		LED_AFF1 = 0; LED_AFF2 = 0; LED_AFF3 = 1; LED_AFF4 = 0; \
		OUTPUT_CHIFFRE(nb); \
	}

#define SET_AFF4(nb) \
	{ \
		LED_AFF1 = 0; LED_AFF2 = 0; LED_AFF3 = 0; LED_AFF4 = 1; \
		OUTPUT_CHIFFRE(nb); \
	}

#define CLEAR_AFF {AFF_PORT = 0;}



// 1 input, 0 output      76543210
#define TRISACONF 0xFF // 11111111
#define TRISBCONF 0xC1 // 11000001
#define TRISDCONF 0x00 // 00000000
#define TRISECONF 0x00 // 00000000

#ifndef DEBUG_RS232
	#define TRISCCONF 0xF9 // 11111001
#else
	#define TRISCCONF	0xF9 // 11111001	=> RC6/7 input !!!
#endif

#endif

